#!usr/bin/perl

#####################################################
#
#  Soldier of Fortune 2: Double Helix - Log Parser
#
#  Author:  Sam Evans (sam@neuroflux.com)
#  Modified:  Jesse 'miCro' Brandeburg (go_jesse@yahoo.com)
#  Modified:  Ravencroft (ravencroft@numbnutz.net)
#
#  Thanks to Jim Walsh for sending the code that fixed the colorizer!
#
# change history:
# Feb 25, 2003 - began changes for time tracking, cleaned up code
#              - tabstops removed, columns 90
#              - all warnings removed (-w option)
#              - updated weapons 
#                - add MOD_M15_GRENADE_ALT
#                - add MOD_SILVER_TALON_ALT
#                - add MOD_TARGET_LASER
#              - new config file options: TOPWEAP, MINIMUM_KILLS, MINIMUM_TIME
#                                         EFF_FILENAME, 
# Feb 28, 2003 - incorporated Ravencroft changes for efficiency sort
# March 7, 2003 - fixed 0h0m0s bug in efficiency stats
# March 9, 2003 - added "real" playtime, when spectator doesn't count towards playtime
#               - better suicide tracking (don't count MOD_TEAMCHANGE)
#               - delete outputdir/players/* each time we run
# April 22, 2003 - fixed a pretty major bug in the history parser that biffed
#                  when reading names with W: or PV: etc in them
# April 30, 2003 - many changes, mostly bug fixes, 
#                  thanks to Minimad for some fixes to stats accuracy
#####################################################

use strict;
use Getopt::Long;
use File::Copy;

# this is typically overridden by the logparse.conf file now
my %TEMPLATE;

#################################################################
                                #
# NOTE DO NOT CHANGE THIS, IT'S CURRENTLY BROKE!
my $WEAPONS_TO_DISPLAY = "all"; # Settings here are either 
                                # all or the Weapons as printed in the log file
                                # Example: MOD_SABER MOD_BLASTER
                                # would only display stats for the SABER and BLASTER weapons
#################################################################

################## Lines we look for in the template ############
my $LineStylesheet      =   "<!-- STYLESHEET -->";      # 
my $LineServerStats     =   "<!-- SERVER STATS -->";    #
my $LinePlayerStats     =   "<!-- PLAYER STATS -->";    # You should not need to change these
my $LinePopularWeapons  =   "<!-- POPULAR WEAPONS -->"; #
my $LineTopWeaponers    =   "<!-- TOP WEAPON USERS -->";#
#################################################################

################## Lines we look for in the template ############
my $I_LinePlayerWeapon    = "<!-- PLAYER WEAPON -->";     # You should not need to change these
my $I_LinePlayerStats     = "<!-- PLAYER STATS -->";      #
my $I_LineStylesheet      = "<!-- STYLESHEET -->";        #
my $I_LinePlayerVPlayer   = "<!-- PLAYER VS PLAYER -->";  #
#################################################################

my %WEAPON_MAP = ( 
################################################################################
# THESE ARE SET BY THE GAME     CHANGE THIS COLUMN TO WHATEVER YOU WANT DISPLAYED #
################################################################################
    "MOD_KNIFE"                     =>  "Knife",
    "MOD_KNIFE_ALT"                 =>  "Knife Throw",
    "MOD_US_SOCOM_PISTOL_ALT"       =>  "SOCOM: Head Smash",
    "MOD_US_SOCOM_PISTOL"           =>  "SOCOM",
    "MOD_M1911A1_PISTOL_ALT"        =>  "M1911-A1: Head Smash",
    "MOD_M1911A1_PISTOL"            =>  "M1911-A1",
    "MOD_SMOHG92_GRENADE"           =>  "Frag Nade",
    "MOD_SMOHG92_GRENADE_ALT"       =>  "Frag Nade Rolled",
    "MOD_M84_GRENADE"               =>  "Flash Nade",
    "MOD_M84_GRENADE_ALT"           =>  "Flash Nade Rolled",
    "MOD_ANM14_GRENADE"             =>  "Fire Nade",
    "MOD_ANM14_GRENADE_ALT"         =>  "Fire Nade Rolled",
    "MOD_M15_GRENADE"               =>  "Smoke Nade",
    "MOD_M15_GRENADE_ALT"           =>  "Smoke Nade Rolled",
    "MOD_M590_SHOTGUN"              =>  "Shottie",
    "MOD_M590_SHOTGUN_ALT"          =>  "Shottie Butt",
    "MOD_USAS_12_SHOTGUN"           =>  "Auto Shottie",
    "MOD_SIG551"                    =>  "SIG551",
    "MOD_SILVER_TALON"              =>  "Silver Talon",
    "MOD_SILVER_TALON_ALT"          =>  "Silver Talon: Head Smash",
    "MOD_AK74_ASSAULT_RIFLE"        =>  "AK74",
    "MOD_AK74_ASSAULT_RIFLE_ALT"    =>  "AK74 Stab",
    "MOD_M4_ASSAULT_RIFLE"          =>  "M4",
    "MOD_M4_ASSAULT_RIFLE_ALT"      =>  "M4 Nade",
    "MOD_TRIGGER_HURT_NOSUICIDE"    =>  "Killed By Dem Bomb",
    "MOD_WATER"                     =>  "Drowned",
    "MOD_TRIGGER_HURT"              =>  "Cratered",
    "MOD_SUICIDE"                   =>  "Suicide",
    "MOD_FALLING"                   =>  "Ground Fall",
    "MOD_CRUSH"                     =>  "Crushed",
    "MOD_TELEFRAG"                  =>  "TeleFraged",
    "MOD_UNKNOWN"                   =>  "Unknown",
    "MOD_TEAMCHANGE"                =>  "Team Change",
    "MOD_RPG7_LAUNCHER"             =>  "RPG Launcher",
    "MOD_MM1_GRENADE_LAUNCHER"      =>  "MM1 Grenade Launcher",
    "MOD_M3A1_SUBMACHINEGUN"        =>  "M3A1 SMG",
    "MOD_MSG90A1_SNIPER_RIFLE"      =>  "Sniper Rifle",
    "MOD_MICRO_UZI_SUBMACHINEGUN"   =>  "Micro Uzi",
    "MOD_M60_MACHINEGUN"            =>  "M60 MG",
    "MOD_MP5"                       =>  "MP5",
    "MOD_TARGET_LASER"              =>  "OSP Admin Strike"
     );


###################################################
# -- PROGRAM -- DO NOT MODIFY BELOW THIS POINT -- #
###################################################
#
##################
# Global Variables
##################
my %players;
my %individuals;
my %totalTime;
my %weapon;
my %victim;
my %player_stats;
my %top_player_weapon;
my %player_vs_player;
my %suicides;
my %filter;
my %by_skill;
my %by_eff;
my $count = 1;
my $uniquePlayers = 0;
my $totalKills = 0;
my $alt = 0;
my $lineCount = 0;
my $noInit = 0;
my ($serverName, $administrator, $adminEmail, $server_stats1, $player_stats1, $weapon_stats1, $popular_weapon1);
my ($stylesheet, $weapon_handler);
my $noLastLine = 0;
my $doNextLine = 0;
my $currentLineNum = 0;
my $processedLines = 0;
my $lastLineNum = 0;
my $i_count = 0;
my ($lastLine, $lastLineToMatch);
my ($DEBUG) = 0;
my ($OUTPUT_FILENAME, $LOGFILE, $OUTPUTDIR, $STYLESHEET, $DATADIR, $REINIT, $TOPNUM, $TOPWEAP, $FILTER, $HELP, $CONFIG_FILE, $PLAYERDATADIR);
my $IMAGESDIR = "images";
my ($SUICIDES_OUTPUT_FILENAME, $DEATHS_OUTPUT_FILENAME, $SKILLS_OUTPUT_FILENAME, $TIMEPLAYED_OUTPUT_FILENAME, $EFF_OUTPUT_FILENAME);
my ($MINIMUM_KILLS, $MINIMUM_TIME);
my ($AdminEmail, $Administrator, $ServerName);

my %COLOR_MAP = (
        "0" =>  "#000000",  # Black
        "1" =>  "#FF0000",  # Red
        "2" =>  "#80FF00",  # Green
        "3" =>  "#FFFF00",  # Yellow
        "4" =>  "#0000FF",  # Blue
        "5" =>  "#00FFFF",  # Cyan
        "6" =>  "#FF00FF",  # Magenta
        "7" =>  "#FFFFFF"   # White
        );

my $VERSION = "1.0.5.4";
my $AUTHOR   = "Sam Evans (<a href=\"mailto:sam\@neuroflux.com\"  class=\"player\">sam\@neuroflux.com<\/a>)";
my $MODIFIED = "Jesse 'miCro' (<a href=\"mailto:go_jesse\@yahoo.com\" class=\"player\">go_jesse\@yahoo.com<\/a>)";
my $MODRC = "Ravencroft (<a href=\"mailto:ravencroft\@numbnutz.net\" class=\"player\">ravencroft\@numbnutz.net<\/a>)";

## VROOM!  Here we go.. ##

GetOptions(
        "c=s"       => \$CONFIG_FILE,
        "logfile=s" => \$LOGFILE,
        "reinit!"   => \$REINIT,
        "d!"        => \$DEBUG,
        "h!"        => \$HELP
      );

if ($HELP) {
    system("perldoc $0");
    exit;
}

########################################
# -- READ IN THE CONFIGURATION FILE -- #
########################################

if (!$CONFIG_FILE) {
    $CONFIG_FILE = "logparse.conf";     # Default Filename
}

if (!-f $CONFIG_FILE) {
    die("Error:  Your Configuration file could not be located.\nError: $!\n\n");
} else {
    open (CONF, $CONFIG_FILE);
    while (<CONF>) {
        if ($_ =~ /^Administrator\s+(.+)$/i) {
            $Administrator = $1;
        }
        if ($_ =~ /^AdminEmail\s+(.+)/i) {
            $AdminEmail = $1;
        }
        if ($_ =~ /^ServerName\s+(.+)/i) {
            $ServerName = $1;
        }
        if ($_ =~ /^TOPNUM\s+(\d+)/) {
            $TOPNUM = $1;
            chomp($TOPNUM);
        }
        if ($_ =~ /^TOPWEAP\s+(\d+)/) {
            $TOPWEAP = $1;
            chomp($TOPWEAP);
        }
        if ($_ =~ /^MINIMUM_KILLS\s+(\d+)/) {
            $MINIMUM_KILLS = $1;
        }
        if ($_ =~ /^MINIMUM_TIME\s+(\d+)/) {
            $MINIMUM_TIME = $1;
        }
        if ($_ =~ /^OUTPUT_FILENAME\s+(.+)/) {
            $OUTPUT_FILENAME = $1;
            chomp($OUTPUT_FILENAME);
            if ($DEBUG) { print "output filename = $OUTPUT_FILENAME\n"; };
        }
        if ($_ =~ /^SKILLS_FILENAME\s+(.+)/) {
            $SKILLS_OUTPUT_FILENAME = $1;
            chomp($SKILLS_OUTPUT_FILENAME);
        }
        if ($_ =~ /^DEATHS_FILENAME\s+(.+)/) {
            $DEATHS_OUTPUT_FILENAME = $1;
            chomp($DEATHS_OUTPUT_FILENAME);
        }
        if ($_ =~ /^EFF_FILENAME\s+(.+)/) {
            $EFF_OUTPUT_FILENAME = $1;
            chomp($EFF_OUTPUT_FILENAME);
        }
        if ($_ =~ /^SUICIDES_FILENAME\s+(.+)/) {
            $SUICIDES_OUTPUT_FILENAME = $1;
            chomp($SUICIDES_OUTPUT_FILENAME);
        }
        if ($_ =~ /^LOGFILE\s+(.+)/) {
            $LOGFILE = $1 if (!$LOGFILE);
            chomp($LOGFILE);
        }
        if ($_ =~ /^TIMEPLAYED_FILENAME\s+(.+)/) {
            $TIMEPLAYED_OUTPUT_FILENAME = $1;
            chomp($TIMEPLAYED_OUTPUT_FILENAME);
        }
        if ($_ =~ /^MAIN_TEMPLATE\s+(.+)/) {
            my $MAINTEMP = $1;
            chomp($MAINTEMP);
            $TEMPLATE{"TEMPLATE1"} = $MAINTEMP;
        }
        if ($_ =~ /^PLAYER_TEMPLATE\s+(.+)/) {
            my $PLAYERTEMP = $1;
            chomp($PLAYERTEMP);
            $TEMPLATE{"TEMPLATE2"} = $PLAYERTEMP;
        }
        if ($_ =~ /^LOGFILE\s+(.+)/) {
            $LOGFILE = $1 if (!$LOGFILE);
            chomp($LOGFILE);
        }
        if ($_ =~ /^OUTPUTDIR\s+(.+)/) {
            $OUTPUTDIR = $1;
            chomp($OUTPUTDIR);
        }
        if ($_ =~ /^PLAYERDATADIR\s+(.+)/) {
            $PLAYERDATADIR = $1;
            chomp($PLAYERDATADIR);
        }
        if ($_ =~ /^DATADIR\s+(.+)/) {
            $DATADIR = $1;
            chomp($DATADIR);
        }
        if ($_ =~ /^STYLESHEET\s+(.+)/) {
            $STYLESHEET = $1;
            chomp($STYLESHEET);
        }
        if ($_ =~ /^FILTER\s+(.+)/) {
            $FILTER = $1;
            chomp($FILTER);
        }
    }
    close(CONF);
}

# Create the configuration mappings for easy stats creation

my %CONF_MAP = (
        "0" =>  "$OUTPUT_FILENAME",
        "1" =>  "$SKILLS_OUTPUT_FILENAME",
        "2" =>  "$DEATHS_OUTPUT_FILENAME",
        "3" =>  "$SUICIDES_OUTPUT_FILENAME",
        "4" =>  "$EFF_OUTPUT_FILENAME",
        "5" =>  "$TIMEPLAYED_OUTPUT_FILENAME",
        );

if ($REINIT) {
    #
    # Reinitialize the data file that stores the user statistics
    #
    init_datafile();
    print "Datafile has been re-initialized.\n";
    exit;
}

foreach my $check_template (sort keys %TEMPLATE) {
    if (!-f $TEMPLATE{$check_template}) {
        print "Error:  Could not open the Template file ($TEMPLATE{$check_template}).\nError: $!\n\n";
        exit;
    }
}

if (!-f $LOGFILE) {
    print "Error:  The log file $LOGFILE could not be found.\nError: $!\n\n";
    exit;
}

if (!-e $OUTPUTDIR) {
    print "creating output directory ($OUTPUTDIR)\n";
    mkdir $OUTPUTDIR,0755 || die "couldn't create $OUTPUTDIR\nError: $!\n\n";
}

if (!-e $OUTPUTDIR."/".$PLAYERDATADIR) {
    print "creating output directory for statistical files ($OUTPUTDIR/$PLAYERDATADIR)\n";
    mkdir $OUTPUTDIR."/".$PLAYERDATADIR,0755 || die "couldn't create $OUTPUTDIR/$PLAYERDATADIR\nError: $!\n\n";
}

if (!-e $OUTPUTDIR."/".$IMAGESDIR) {
    print "creating images directory ($OUTPUTDIR/$IMAGESDIR)\n";
    mkdir $OUTPUTDIR."/".$IMAGESDIR,0755 || die "couldn't create $OUTPUTDIR/$IMAGESDIR\nError: $!\n\n";
}
print "copying files to $IMAGESDIR\n";
copy ("left.gif",$OUTPUTDIR."/".$IMAGESDIR."/left.gif")  || die "couldn't copy left.gif\nError: $!\n\n";
copy ("right.gif",$OUTPUTDIR."/".$IMAGESDIR."/right.gif")|| die "couldn't copy right.gif\nError: $!\n\n";
copy ("main.gif",$OUTPUTDIR."/".$IMAGESDIR."/main.gif")  || die "couldn't copy main.gif\nError: $!\n\n";
copy ("sof2logger.gif",$OUTPUTDIR."/".$IMAGESDIR."/sof2logger.gif")  || die "couldn't copy sof2logger.gif\nError: $!\n\n";

if (!-e $OUTPUTDIR."/stylesheets") {
    print "creating stylesheets directory ($OUTPUTDIR/stylesheets)\n";
    mkdir $OUTPUTDIR."/stylesheets",0755 || die "couldn't create $OUTPUTDIR/stylesheets\nError: $!\n\n";
    print "copying $STYLESHEET to stylesheets\n";
    copy ($STYLESHEET,$OUTPUTDIR."/stylesheets/".$STYLESHEET)  || die "couldn't copy $STYLESHEET\nError: $!\n\n";
}

if (!-e $DATADIR) {
    print "creating the DATADIR ($DATADIR)\n";
    mkdir $DATADIR,0755 || die "couldn't create $DATADIR\nError: $!\n\n";
}

if (-f $FILTER) {
    # Read in the names in this list, push them into the filter
    open (FILTER, $FILTER) or die("Error:  Could not open your filter file\nError: $!\n\n");
    while (<FILTER>) {
        chomp($_);
        $filter{$_}++;
    }
    close(FILTER);
}

#empty out the $PLAYERDATADIR because all this script could leave would be old data
opendir (PLAYERDIR, $OUTPUTDIR."/".$PLAYERDATADIR);
foreach my $playerfile (readdir PLAYERDIR) {
    if ($playerfile !~ /^.$/ && $playerfile !~ /^..$/) {
        if ($DEBUG) { print "deleting $playerfile from $OUTPUTDIR/$PLAYERDATADIR\n"; }
        unlink $OUTPUTDIR."/".$PLAYERDATADIR."/".$playerfile;
    }
}
closedir PLAYERDIR;

if (!-f "$DATADIR/players.dat") {
    init_datafile();
} else {
    # Pull existing values into the appropriate hashes
    open (PLAYERFILE, "$DATADIR/players.dat");
    while (<PLAYERFILE>) {
        if ($_ =~ /^LastNum\:\s(\d+)/) {
            $lastLineNum = $1;
            next;
        }
        if ($_ =~ m/^LastLine\:\s(.+)/){
            $lastLineToMatch = $1;
            $lastLineToMatch =~ tr/\\//d;
            next;
        }
        if ($_ =~ /^P\:(.+)\t(\d+)\t(\d+)/) {    # Player Stats (Name, Kills, Deaths)
            $players{$1} = $2 if (!$filter{$1});
            $victim{$1} = $3 if (!$filter{$1});
            next;
        }
        if ($_ =~ /^W\:(.+)\t(\d+)/) {       # Weapon Stats (Weapon, # Kills)
            $weapon{$1} = $2;
            next;
        }
        if ($_ =~ /^TP\:(.+)\t(.+)\t(\d+)/) {    # Top Weapons (Player, Weapon, # Kills)
            $top_player_weapon{$1."\t".$2} = $3 if (!$filter{$1});
            next;
        }
        if ($_ =~ /^PV:(.+)\t(.+)\t(\d+)/) {     # Player, Victim, Kills
            $player_vs_player{$1."\t".$2} = $3;
            next;
        }
        if ($_ =~ /^S:(.+)\t(\d+)/) {    # Player, # Suicides
            $suicides{$1} = $2 if (!$filter{$1});
            next;
        }
        if ($_ =~ /^TK\:\s(\d+)/) {  # Total Kills
            $totalKills = $1;
            next;
        }
        if ($_ =~ /^TM\:(.+)\t(\d+)/) {  # Total Time played on server (Name, Time (seconds))
            $totalTime{$1} = $2;
            next;
        }
#        if ($_ =~ /(\sby\s|\skilled\s|\<world\>)/) {
#            #look for bad names
#            print "datafile at $. contains strange line $_\n";
#        }
    }
    close(PLAYERFILE);
}

read_logfile(0); # First pass.  This looks for the same existing logfile so it can add info to the hash

if ($processedLines == 0 && $noLastLine == 0) {
    if ($DEBUG) {
        # The logfile must have rotated since we last read it, call the read_logfile again
        print "Could not find the last line in the logfile\n";
        <STDIN>;
    }
    read_logfile(999);
}

sub read_logfile {
    $currentLineNum = 0;
    my $timekeeper = 0;
    my $flag = shift;
    my %playerjoin;
    keys(%playerjoin) = 32; #set up a cache of this many
    my %playername;
    keys(%playername) = 32;
    my %playerteam;
    keys(%playerteam) = 32;
    open (LOG, $LOGFILE) or die("Could not open the Logfile ($LOGFILE) for parsing.\nError: $!");

    if ($DEBUG) {
        print "DEBUG: lastLineNum = $lastLineNum\n";
    }

    while (<LOG>) {
        $currentLineNum++ if (!eof(LOG));
        if ($doNextLine != 1) {
            if ($lastLineNum == 0) {    # There was not a Last Line # in the player.dat file, start from scratch
                $doNextLine = 1;
            } else {
                
                if ($currentLineNum == $lastLineNum) {
                    if ($DEBUG) {
                        print "Found the last line of the logfile\n";
                    }
                    $noLastLine = 1;
                    $doNextLine = 1;
                }
            }
        }
        if ($doNextLine == 1 || $flag == 999 && !eof(LOG)) {
            $processedLines++;
            my ($player1, $player2, $weapon_users);
            # Kill: 1022 21 32: <world> killed VidCow by MOD_CRUSH
            
            #count suicides out of this line
            #3588:24 Kill: 7 7 25: FuManchu killed FuManchu by MOD_SUICIDE
            if ($_ =~ /Kill\:\s(\d+)\s(\d+)\s(\d+)\:\s(.+?)\skilled\s(.+)\sby\s(MOD_[0-9A-Z_]+)/) {
                $suicides{$5} = 0 unless ($suicides{$5});
                if ($1 == 1022 && $3 != 29) {   # World killed this player, not by MOD_TRIGGER_HURT_NOSUICIDE (demo bomb)
                    $suicides{$5}++;
                }
                #check for teamchange events and ignore them
                if (($3 != 26) && (($3 == 25) || ($1 == $2))) {
                    $suicides{$5}++;
                }
            }

            # check to see if there might be a corrupt log file
            if ($DEBUG) {
                if ($_ =~ /----------/ ) { $timekeeper = 0; }
                elsif ($_ =~ /Shutdown/) { $timekeeper = 0; }
                elsif ($_ =~ /InitGame/) { $timekeeper = 0; }
                elsif ($_ =~ /^\s*(\d+)\:(\d\d)/) {
                    my $timekeepernew = ($1 * 60) + $2;
                    if  ($timekeepernew < $timekeeper ) {
                        print "line $. wierd time\n";
                    }
                    $timekeeper = $timekeepernew;
                }
            }

            
#regular kills
#3015:21 Kill: 14 13 11: =HBurg= Onatop killed Beetlebub by MOD_AK74_ASSAULT_RIFLE
#yuck, the player named 'killed by' really messed things up here, but its fixed now
            if ($_ =~ /Kill\:\s(\d+)\s(\d+)\s(\d+)\:\s(.+?)\skilled\s(.+)\sby\s(MOD_[0-9A-Z_]+)/) {
             if (($3 != 26) && ($1 != $2)) { # not MOD_TEAMCHANGE based suicide
                $totalKills++;
                $player1 = $playername{$1};
                $player2 = $playername{$2};
        
                $weapon_users = $6;
                chomp($weapon_users);

                print "Unknown weapon parsed $6\n" unless (defined($WEAPON_MAP{$6}));
    
                $victim{$player2}++ if (!$filter{$player2} || ($1 != $2));
                
                if (!$filter{$player1}) {
                    #update kills for this player
                    $players{$player1}++ if ($1 != $2);
                }
        
                if ($weapon_users) {
                    $weapon{$weapon_users}++;
                }

                $top_player_weapon{$player1."\t".$weapon_users}++ if (!$filter{$player1});
                $player_vs_player{$player1."\t".$player2}++ if ($1 != $2);

                #we're done with Kill: lines so lets just exit parsing this line
                next;
             }
            }
            #lets try to count time played
#time notification starts with
#5953:23 ClientConnect: 0 - 3.15.8.2:20100 [GUIDGUIDGUIDGUIDGUIDGUIDGUIDGUID]
#5953:23 ClientUserinfoChanged: 0 n\Pigeon\t\1\identity\snowindoor

#connect is only marginally useful at this point
#            if ($_ =~ /^\s*(\d+)\:(\d\d)\s*ClientConnect\:\s(\d+)\s/) {
#                my $timejoin = (($1 * 60) +  $2); #get value in seconds
#                $playerjoin{$3} = $timejoin;
##               if ($DEBUG) { print "Client $3 connect at $playerjoin{$3}\n"; };
#                next;
#            }

            #note: \t\0 = (deathmatch, noteam), \t\1 = Red, \t\2 = Blue, \t\3 = Spectator
            if ($_ =~ /^\s*(\d+)\:(\d\d)\s*ClientUserinfoChanged\:\s(\d+)\sn\\(.+?)\\t\\(\d)/) {
                my $timejoin = (($1 * 60) +  $2); #get value in seconds
                my $newteam = $5;
                #fill in the slot with a name
                $playername{$3} = $4;
                unless (defined($playerteam{$3})) {
                    $playerteam{$3} = $newteam;
                    if ($newteam != 3) {
                        $playerjoin{$3} = $timejoin;
                    }
                    else { #player went straight to spec
                        $playerjoin{$3} = 0;
                    }
                }
                else { # user/slot already being tracked
                    # if the player's team hasn't changed, do nothing
                    # otherwise...
                    if ($newteam != $playerteam{$3}) {
                        if ($newteam != 3) {
                            unless ($playerjoin{$3}) {
                                $playerjoin{$3} = $timejoin;
                            }
                        }
                        else { #player joined spectators, compute time
                            my $timeexit = (($1 * 60) + $2); #get value in seconds
                            $totalTime{$4} += ($timeexit - $playerjoin{$3});
                            #clean up the slot tables
                            delete $playerteam{$3};
                            delete $playerjoin{$3};
                            delete $playername{$3};
                        }
                    }
                }
                next;
            }

## another example
##  99:55 ClientConnect: 4 - 6.3.2.14:19246 [GUID]
##  99:55 ClientUserinfoChanged: 4 n\FrancoV\t\2\identity\thug2
## 100:03 ClientDisconnect: 4
            if ($_ =~ /^\s*(\d+)\:(\d\d)\s*ClientDisconnect\:\s(\d+)/) {
                #if there is a disconnect with no connect in the log
                next unless ($playerjoin{$3});
                my $timeexit = (($1 * 60) + $2); #get value in seconds
                if (!$filter{$playername{$3}}) {
                    #initialize the user's data if need be
                    $totalTime{$playername{$3}} = 0 unless (defined($totalTime{$playername{$3}}));
                    $totalTime{$playername{$3}} += ($timeexit - $playerjoin{$3});
                }
#               if ($DEBUG) { print "Client $3 disconnect at $timeexit, totaltime now $totalTime{$playername{$3}}\n"; };    
                #clean up the slot tables
                delete $playerteam{$3};
                delete $playerjoin{$3};
                delete $playername{$3};
                next;
            }

#   1:32 ShutdownGame:
            #finish up any unclosed player times, for each round
            #fixup with any line that has ShutdownGame
            if ($_ =~ /^\s*(\d+)\:(\d\d)\s*ShutdownGame\:/ ) {
                my $timeexit = (($1 * 60) + $2); #get value in seconds
                foreach my $ind (keys %playerjoin) {
                    # don't compute time if join is 0 (special spectator case)
                    if ($playerjoin{$ind} && !$filter{$playername{$ind}} ) {
                        #initialize the user's data if need be
                        $totalTime{$playername{$ind}} = 0 unless (defined($totalTime{$playername{$ind}}));
                        if ($timeexit >= $playerjoin{$ind}) {
                            $totalTime{$playername{$ind}} += ($timeexit - $playerjoin{$ind});
                            if ($DEBUG && ($totalTime{$playername{$ind}} < 0)) { print "BAD LOGFILE? line $. introduced negative number for $playername{$ind}\n" };
                        }
                    }
                    #clean up the slot tables
                    delete $playerteam{$ind};
                    delete $playerjoin{$ind};
                    delete $playername{$ind};
                }
                next;
            }

            #fix up slots to be safe at every InitGame!!!
            # solves problem where some logs are missing ShutdownGame (corrupt log)
            if ($_ =~ /^\s*(\d+)\:(\d\d)\s*InitGame\:/ ) {
                foreach my $ind (keys %playerjoin) {
                    #clean up the slot tables
                    delete $playerteam{$ind};
                    delete $playerjoin{$ind};
                    delete $playername{$ind};
                }
            }
                    
        }
    }
    continue {
        #this is executed once for each while pass even if we 'next'
        $lastLine = $_;
    }

    close(LOG);
    if ($currentLineNum != $lastLineNum) {  
        $lastLineNum = $currentLineNum;
    }
    $uniquePlayers = scalar keys %players;
}

############################################################################
# Write the hash information to the player datafile before we munge the data
############################################################################
open (PLAYEROUT, ">$DATADIR/players.dat") or die("Could not open the player datafile ($DATADIR/players.dat) for writing.\nError: $!");
print PLAYEROUT "Init: ". scalar localtime()."\n";
print PLAYEROUT "LastNum: $lastLineNum\n";
print PLAYEROUT "LastLine: $lastLine\n";
print PLAYEROUT "TK: $totalKills\n";
foreach my $p (sort { $players{$b} <=> $players{$a} } keys %players) {
    $victim{$p} = 0 unless (defined($victim{$p}));
    $players{$p} = 0 unless (defined($players{$p}));
    print PLAYEROUT "P:$p\t$players{$p}\t$victim{$p}\n";

    # Write out the player's profile
    my $individual_player = $p;
    $individual_player =~ s/(\W)//g;
}
foreach my $w (sort { $weapon{$b} <=> $weapon{$a} } keys %weapon) {
    print PLAYEROUT "W:$w\t$weapon{$w}\n";
}
foreach my $tp (sort { $top_player_weapon{$b} <=> $top_player_weapon{$a} } keys %top_player_weapon) {
    my ($sp1, $sp2) = split(/\t/, $tp);
    print PLAYEROUT "TP:$sp1\t$sp2\t$top_player_weapon{$tp}\n";
}
foreach my $pv (sort { $player_vs_player{$b} <=> $player_vs_player{$a} } keys %player_vs_player) {
    my ($pp1, $pp2) = split(/\t/, $pv);
    print PLAYEROUT "PV:$pp1\t$pp2\t$player_vs_player{$pv}\n";
}
foreach my $su (sort { $suicides{$b} <=> $suicides{$a} } keys %suicides) {
    print PLAYEROUT "S:$su\t$suicides{$su}\n";
}
foreach my $p (sort { $totalTime{$b} <=> $totalTime{$a} } keys %totalTime) {
    print PLAYEROUT "TM:$p\t$totalTime{$p}\n";
}

close(PLAYEROUT);


sub make_files ($) {
    my $stat_to_make = shift;
    $server_stats1 = server_stats();
    $player_stats1 = player_stats($stat_to_make); # Create the main stats page
    $popular_weapon1 = popular_weapon();
    $stylesheet = stylesheet();
    $weapon_handler = top_weapon_handler();

    open(HTMLINDEX, ">$OUTPUTDIR/$CONF_MAP{$stat_to_make}") or die("Could not open the output file: $OUTPUTDIR/$CONF_MAP{$stat_to_make} for writing\n");
    open (TEMPLATE, $TEMPLATE{TEMPLATE1});
    while (<TEMPLATE>) {
        $_ =~ s/$LineStylesheet/$stylesheet/g;
        $_ =~ s/$LineServerStats/$server_stats1/g;
        $_ =~ s/$LinePlayerStats/$player_stats1/g;
        $_ =~ s/$LineTopWeaponers/$weapon_handler/g;
        $_ =~ s/$LinePopularWeapons/$popular_weapon1/g;
        print HTMLINDEX $_;
    }
    close (HTMLINDEX);
}

#prune players without enough kills/time
#we save the data just above this, so people can get rated eventually
print "limiting players shown to kills > $MINIMUM_KILLS, time > $MINIMUM_TIME\n";
$count = 0;
# # this table is a pain because it uses concatentated key field ($player\t$player)
# foreach my $prune (keys %player_vs_player) {
# {
#     my ($plyr, $vctm) = split(/\t/, $prune);
#     if ($players{$plyr} <= $MINIMUM_KILLS) {
#         delete $player_vs_player{$prune};
#         $count++;
#     }
# }
# # this table is a pain because it uses concatentated key field ($player\t$weapon)
# foreach my $prune (keys $top_player_weapon) {
#     my ($plyr, $weap) = split (/\t/, $prune);
#     if ($players{$plyr} <= $MINIMUM_KILLS) {
#         delete $top_player_weapon{$prune};
#         $count++;
#     }
# }
foreach my $prune (keys %players) {
    if ($players{$prune} < $MINIMUM_KILLS) {
        if ($DEBUG) { print "player $prune kills $players{$prune} < $MINIMUM_KILLS\n"; }
        delete $players{$prune};
        delete $victim{$prune};
        delete $suicides{$prune};
        delete $totalTime{$prune};
        $count++;
    } 
}
print "removed $count players for < Minimum Kills\n";
$count = 0;
foreach my $prune (keys %totalTime) {
    if ($totalTime{$prune} < $MINIMUM_TIME) {
        if ($DEBUG) { print "player $prune time $totalTime{$prune} < $MINIMUM_TIME\n"; }
        delete $totalTime{$prune};
        delete $players{$prune};
        delete $victim{$prune};
        delete $suicides{$prune};
        $count++;
    }
}
print "removed $count players for < Minimum Time\n";
$count = 1;

########################################################################
# Pull out the template contents, replace the tags with actual content #
########################################################################

make_files(0); #kills
make_files(1); #deaths
make_files(2); #suicides
make_files(3); #skill

#update efficiency for each player
foreach my $top_players (keys %players) {
    #total = $kills + $deaths + $suicides
    $players{$top_players} = 0 unless (defined($players{$top_players}));
    $victim{$top_players} = 0 unless (defined($victim{$top_players}));
    $suicides{$top_players} = 0 unless (defined($suicides{$top_players}));

    my $total = ($players{$top_players} + $victim{$top_players} + $suicides{$top_players});
    #( kills *100 )/ total = eff
    if ($total > 0 && $players{$top_players} > 0) {
        my $eff = ( 100 * $players{$top_players} ) / $total;
        $by_eff{$top_players} = $eff;
    } else {
        $by_eff{$top_players} = 0;
    }
}

make_files(4); #new for efficiency
make_files(5); # new for time tracking
    
###########################################
# Generate the individual stats pages now #
###########################################
foreach my $i_player (keys %individuals) {
    my $i_stylesheet = rel_stylesheet();
    my $i_playervplayer = I_player_vs_player($i_player);
    my $i_playerweapon = I_popular_weapon($i_player);
    my $i_playerstats = I_player_stats($i_player);
    # Write out the player's profile
    my $i_player1 = $i_player;
    $i_player1 =~ s/(\W)//g;

    #fix up special case names
    if ($i_player1 eq "") {
        # try again
        $i_player1 =~ s/\./_dot_/g;
        $i_player1 =~ s/\s/_space_/g;
        $i_player1 =~ s/\!/_bang_/g;
        $i_player1 =~ s/\@/_at_/g;
        $i_player1 =~ s/\#/_pound_/g;
        $i_player1 =~ s/\$/_dollar_/g;
        $i_player1 =~ s/\%/_percent_/g;
        $i_player1 =~ s/\^/_hat_/g;
        $i_player1 =~ s/\&/_amp_/g;
        $i_player1 =~ s/\*/_star_/g;
        $i_player1 =~ s/\(/_op_/g;
        $i_player1 =~ s/\)/_cp_/g;
        $i_player1 =~ s/(\W)//g;
        if ($DEBUG) { print "name converted to $i_player1 from $i_player\n"; }
    }

    open (PLAYERHTMLOUT, ">$OUTPUTDIR/$PLAYERDATADIR/$i_player1.html") or die("Could not open the individual player file ($OUTPUTDIR/$PLAYERDATADIR/$i_player1.html) for writing.\nError: $!\n\n");
    open (TEMPLATE2, $TEMPLATE{TEMPLATE2});
        while (<TEMPLATE2>) {
            $_ =~ s/$I_LinePlayerWeapon/$i_playerweapon/g;
            $_ =~ s/$I_LineStylesheet/$i_stylesheet/g;
            $_ =~ s/$I_LinePlayerVPlayer/$i_playervplayer/g;
            $_ =~ s/$I_LinePlayerStats/$i_playerstats/g;
            print PLAYERHTMLOUT $_;
        }
    close(TEMPLATE2);
    close(PLAYERHTMLOUT);
        $i_count++;
}
#####################
# Parse the data now
#####################
sub stylesheet {
    return("<link rel=\"stylesheet\" href=\"stylesheets/$STYLESHEET\">\n");
}

sub rel_stylesheet {
    return("<link rel=\"stylesheet\" href=\"../stylesheets/$STYLESHEET\">\n");
}

###################
# Server Statistics
###################
sub server_stats {
    my $totalServertime;
    foreach my $pct (keys %totalTime) {
        $totalServertime += $totalTime{$pct};
    }
    my $hours = int($totalServertime / 3600);    #get hours
    $totalServertime %= 3600;               #remove those seconds that were converted
    my $timeString = sprintf "%5dh%4dm%02ds", $hours, int($totalServertime/60),$totalServertime%60;

    $hours = int($MINIMUM_TIME / 3600);
    $totalServertime = $MINIMUM_TIME % 3600;
    my $minutes = int($totalServertime / 60);

    my $server_stats = "
    <table border=\"1\" cellspacing=\"0\" cellpadding=\"4\" width=\"50%\">\n
    <tr>\n
    <td class=\"header\" colspan=\"2\"><strong><a href=\"http:\/\/jk2.gamersimpact.com\" class=\"player\">SoF2 Logger<\/a> v$VERSION by $AUTHOR<\/td>\n 
    </tr>\n
    <tr>\n
    <td class=\"header\" colspan=\"2\"><strong>Modified by $MODIFIED<\/td>\n    
    <\/tr>\n
    <tr>\n
    <td class=\"header\" colspan=\"2\"><strong>Modified by $MODRC<\/td>\n
    <\/tr>\n
    <tr>\n
    <td class=\"header\" colspan=\"2\"><center><strong>$ServerName Statistics<\/strong></center><\/td>\n
    <\/tr>\n
    <tr>\n
    <td class=\"server-stat\">Server Administrator:<\/td><td class=\"server-value\">$Administrator \[<a href=\"mailto:$AdminEmail\" class=\"player\">$AdminEmail<\/a>\]<\/td>\n
    <\/tr>\n
    <tr>\n
    <td class=\"server-stat\">Total Kills:<\/td><td class=\"server-value\">$totalKills<\/td>\n
    <\/tr>\n
    <tr>\n
    <td class=\"server-stat\">Total Unique Players:<\/td><td class=\"server-value\">$uniquePlayers<\/td>\n
    <\/tr>\n
    <tr>\n
    <td class=\"server-stat\">Total Time Played:<\/td><td class=\"server-value\">$timeString<\/td>\n
    <\/tr>\n
    <tr>\n
    <td class=\"server-stat\"># of Logfile Lines Processed:<\/td><td class=\"server-value\">$currentLineNum<\/td>\n
    <\/tr>\n
    <tr>\n
    <td class=\"server-stat\">Stats Last Updated:<\/td><td class=\"server-value\">". scalar localtime() ."<\/td>\n
    <\/tr>\n
    <tr>\n
    <td class=\"server-stat\"><b>Kill Limit</b> &gt; $MINIMUM_KILLS  <\/td><td class=\"server-value\"><b>Time Limit</b> &gt; $hours hrs $minutes mins <\/td>\n
    <\/tr>\n
    <\/table>\n
    <P>&nbsp;\n
    ";
    return ($server_stats);
}

####################
# Player Statistics 
####################
sub player_stats ($) {
    my $stat_type = shift;
    
    my $playerstring = "
    <table border=\"1\" cellspacing=\"0\" cellpadding=\"0\" width=\"80%\">\n
    <tr>\n
    <td class=\"header\" colspan=\"7\"><center>Top $TOPNUM SOF2 PLAYERS on this server</center><\/td>\n
    <\/tr>\n
    <tr>\n";

    if ($stat_type == 0) {  # Default stat page
        if ($DEBUG) { print "Making default stat page\n"; };    
        $playerstring .= "<td class=\"header\"><strong>Player Name<\/strong><\/td>
                 <td class=\"header\"><strong># Kills<\/strong><\/td>
                 <td class=\"header\"><strong><a href=\"$CONF_MAP{2}\" class=\"player\"># Deaths<\/a><\/strong><\/td>
                 <td class=\"header\"><strong><a href=\"$CONF_MAP{3}\" class=\"player\">Suicides<\/a><\/strong><\/td>
                 <td class=\"header\"><strong><a href=\"$CONF_MAP{1}\" class=\"player\">Skill<\/a><\/strong><\/td>
                 <td class=\"header\"><strong><a href=\"$CONF_MAP{5}\" class=\"player\">Time<\/a><\/strong><\/td>
                 <td class=\"header\"><strong><a href=\"$CONF_MAP{4}\" class=\"player\">Efficiency Ratio<\/a><\/strong><\/td>\n
                 <\/tr>\n";

        foreach my $top_players (sort { $players{$b} <=> $players{$a} } keys %players) {
            if ($count != $TOPNUM + 1) {
                my $kills = $players{$top_players};
                my $suicides = $suicides{$top_players};
                my $deaths = $victim{$top_players};

                $kills = 0 if (!defined($players{$top_players}));
                $suicides = 0 if (!defined($suicides{$top_players}));
                $deaths = 0 if (!defined($victim{$top_players}));
    
                my $img_width;
                my $ratio;
                my $individual_player = $top_players;
                $individual_player =~ s/(\W)//g;

                $playerstring .= "<tr class=\"".alt_colors()."\">\n
                <td>$count\. <a href=\"$PLAYERDATADIR/$individual_player.html\" class=\"player-stat-link\">".strip_colors($top_players)."<\/a><\/td><td>$kills<\/td>";

                my $total = $kills + $deaths + $suicides;
                my $skill = $kills - $deaths - $suicides;

                # yuck this only shows skill for the top TOPNUM players
                $by_skill{$top_players} = $skill;
    
                if ($total > 0 && $kills > 0) {
                    $img_width = ( 100 * $kills ) / $total;
                    $ratio = sprintf "%.2f%%", ($img_width);
                } else {
                    $img_width = "1";
                    $ratio = "0.00%";
                }
    
                $playerstring .= "<td>$deaths<\/td><td>$suicides<\/td><td>$skill<\/td>\n";

                my $seconds = $totalTime{$top_players};
                $seconds = 0 if (!defined($totalTime{$top_players}));
                my $hours = int($seconds / 3600);      #convert to hours
                $seconds %= 3600;               #remove the hours we figured out
                $playerstring .= "<td>";
                $playerstring .= sprintf "%3dh%4dm%02ds",($hours),int($seconds/60),($seconds%60);
                $playerstring .= "<\/td>\n"; 

                $playerstring .= "
                <td><img src=\"$IMAGESDIR\/left.gif\" height=\"10\"><img src=\"$IMAGESDIR\/main.gif\" 
                width=\"$img_width\" height=\"10\"><img src=\"$IMAGESDIR\/right.gif\" 
                height=\"10\">&nbsp;&nbsp;&nbsp;$ratio<\/td>\n
                <\/tr>\n";

                # store useful stats per important player?
                # save this player for individual stats later
                $individuals{$top_players} = 1;
                $count++;
            } else {
                next;
            }
        }
        $count = 1;
        $playerstring .= "<\/table>\n";
        return ($playerstring);
    }

    if ($stat_type == 1) {  # Sort by skill rating
         $playerstring .= "<td class=\"header\"><strong>Player Name<\/strong><\/td>
                 <td class=\"header\"><strong><a href=\"$OUTPUT_FILENAME\" class=\"player\"># Kills<\/a><\/strong><\/td>
                 <td class=\"header\"><strong><a href=\"$CONF_MAP{2}\" class=\"player\"># Deaths<\/a><\/strong><\/td>
                 <td class=\"header\"><strong><a href=\"$CONF_MAP{3}\" class=\"player\">Suicides<\/a><\/strong><\/td>
                 <td class=\"header\"><strong>Skill<\/strong><\/td>
                 <td class=\"header\"><strong><a href=\"$CONF_MAP{5}\" class=\"player\">Time<\/a><\/strong><\/td>
                 <td class=\"header\"><strong><a href=\"$CONF_MAP{4}\" class=\"player\">Efficiency Ratio<\/a><\/strong><\/td>\n
            <\/tr>\n
            ";      
        foreach my $top_players (sort { $by_skill{$b} <=> $by_skill{$a} } keys %by_skill) {
            if ($count != $TOPNUM + 1) {
                my $kills = $players{$top_players};
                my $suicides = $suicides{$top_players};
                my $deaths = $victim{$top_players};

                $kills = 0 if (!defined($players{$top_players}));
                $suicides = 0 if (!defined($suicides{$top_players}));
                $deaths = 0 if (!defined($victim{$top_players}));

                my $img_width;
                my $ratio;
                my $individual_player = $top_players;
                $individual_player =~ s/(\W)//g;

                $playerstring .= "<tr class=\"".alt_colors()."\">\n
                <td>$count\. <a href=\"$PLAYERDATADIR/$individual_player.html\" class=\"player-stat-link\">".strip_colors($top_players)."<\/a><\/td><td>$kills<\/td>";

                my $total = $kills + $deaths + $suicides;

                if ($total > 0 && $kills > 0) {
                    $img_width = ( 100 * $kills ) / $total;
                    $ratio = sprintf "%.2f%%", ($img_width);
                } else {
                    $img_width = "1";
                    $ratio = "0.00%%";
                }

                $playerstring .= "<td>$deaths<\/td><td>$suicides<\/td><td>$by_skill{$top_players}<\/td>\n";

                my $seconds = $totalTime{$top_players};
                $seconds = 0 if (!defined($totalTime{$top_players}));
                my $hours = int($seconds / 3600);      #convert to hours
                $seconds %= 3600;                   #remove the hours we figured out
                $playerstring .= "<td>";
                $playerstring .= sprintf "%3dh%4dm%02ds",($hours),int($seconds/60),($seconds%60);
                $playerstring .= "<\/td>\n"; 

                $playerstring .= "
                <td><img src=\"$IMAGESDIR\/left.gif\" height=\"10\"><img src=\"$IMAGESDIR\/main.gif\" width=\"$img_width\" height=\"10\"><img src=\"$IMAGESDIR\/right.gif\" height=\"10\">&nbsp;&nbsp;&nbsp;$ratio<\/td>\n
                <\/tr>\n";
        
                # save this player for individual stats later
                $individuals{$top_players} = 1;
                $count++;
            } else {
                next;
            }
        } 
        $count = 1;
        $playerstring .= "<\/table>\n";
        return ($playerstring);
    }
    
    if ($stat_type == 2) {  # Sort by number of deaths
         $playerstring .= "<td class=\"header\"><strong>Player Name<\/strong><\/td>
                 <td class=\"header\"><strong><a href=\"$OUTPUT_FILENAME\" class=\"player\"># Kills<\/a><\/strong><\/td>
                 <td class=\"header\"><strong># Deaths<\/strong><\/td>
                 <td class=\"header\"><strong><a href=\"$CONF_MAP{3}\" class=\"player\">Suicides<\/a><\/strong><\/td>
                 <td class=\"header\"><strong><a href=\"$CONF_MAP{1}\" class=\"player\">Skill<\/a><\/strong><\/td>
                 <td class=\"header\"><strong><a href=\"$CONF_MAP{5}\" class=\"player\">Time<\/a><\/strong><\/td>
                 <td class=\"header\"><strong><a href=\"$CONF_MAP{4}\" class=\"player\">Efficiency Ratio<\/a><\/strong><\/td>\n
            <\/tr>\n
            ";     
        foreach my $top_players (sort { $victim{$b} <=> $victim{$a} } keys %victim) {
            if ($count != $TOPNUM + 1) {
                my $kills = $players{$top_players};
                my $suicides = $suicides{$top_players};
                my $deaths = $victim{$top_players};

                $kills = 0 if (!defined($players{$top_players}));
                $suicides = 0 if (!defined($suicides{$top_players}));
                $deaths = 0 if (!defined($victim{$top_players}));

                my $img_width;
                my $ratio;
                my $individual_player = $top_players;
                $individual_player =~ s/(\W)//g;

                $playerstring .= "<tr class=\"" . alt_colors();
                $playerstring .= "\">\n<td>$count\. ";
                $playerstring .= "<a href=\"$PLAYERDATADIR/$individual_player.html\" 
                                 class=\"player-stat-link\">".strip_colors($top_players);
                $playerstring .= "<\/a><\/td><td>$kills<\/td>";

                my $total = $kills + $deaths + $suicides;
                my $skill = $kills - $deaths - $suicides;

                if ($total > 0 && $kills > 0) {
                    $img_width = ( 100 * $kills ) / $total;
                    $ratio = sprintf "%.2f%%", ($img_width);
                } else {
                    $ratio = "0.00%%";
                    $img_width = "1";
                }
                $playerstring .= "<td>$deaths<\/td><td>$suicides<\/td><td>$skill<\/td>\n";

                my $seconds = $totalTime{$top_players};
                $seconds = 0 if (!defined($totalTime{$top_players}));
                my $hours = int($seconds / 3600);      #convert to hours
                $seconds %= 3600;               #remove the hours we figured out
                $playerstring .= "<td>";
                $playerstring .= sprintf "%3dh%4dm%02ds",($hours),int($seconds/60),($seconds%60);
                $playerstring .= "<\/td>\n"; 

                $playerstring .= "
                <td><img src=\"$IMAGESDIR\/left.gif\" height=\"10\"><img src=\"$IMAGESDIR\/main.gif\" width=\"$img_width\" height=\"10\"><img src=\"$IMAGESDIR\/right.gif\" height=\"10\">&nbsp;&nbsp;&nbsp;$ratio<\/td>\n
                <\/tr>\n";

                # save this player for individual stats later
                $individuals{$top_players} = 1;
                $count++;
            } else {
                next;
            }
        }
        $count = 1;
        $playerstring .= "<\/table>\n";
        return ($playerstring);
    }
    if ($stat_type == 3) {  # Sort by number suicides
         $playerstring .= "<td class=\"header\"><strong>Player Name<\/strong><\/td>
                 <td class=\"header\"><strong><a href=\"$OUTPUT_FILENAME\" class=\"player\"># Kills<\/a><\/strong><\/td>
                 <td class=\"header\"><strong><a href=\"$CONF_MAP{2}\" class=\"player\"># Deaths<\/a><\/strong><\/td>
                 <td class=\"header\"><strong>Suicides<\/strong><\/td>
                 <td class=\"header\"><strong><a href=\"$CONF_MAP{1}\" class=\"player\">Skill<\/a><\/strong><\/td>
                 <td class=\"header\"><strong><a href=\"$CONF_MAP{5}\" class=\"player\">Time<\/a><\/strong><\/td>
                 <td class=\"header\"><strong><a href=\"$CONF_MAP{4}\" class=\"player\">Efficiency Ratio<\/a><\/strong><\/td>\n
            <\/tr>\n
            ";     
        foreach my $top_players (sort { $suicides{$b} <=> $suicides{$a} } keys %suicides) {
            if ($count != $TOPNUM + 1) {
                my $kills = $players{$top_players};
                my $suicides = $suicides{$top_players};
                my $deaths = $victim{$top_players};

                $kills = 0 if (!defined($players{$top_players}));
                $suicides = 0 if (!defined($suicides{$top_players}));
                $deaths = 0 if (!defined($victim{$top_players}));

                my $img_width;
                my $ratio;
                my $individual_player = $top_players;
                $individual_player =~ s/(\W)//g;

                $playerstring .= "<tr class=\"".alt_colors()."\">\n
                <td>$count\. <a href=\"$PLAYERDATADIR/$individual_player.html\" class=\"player-stat-link\">".strip_colors($top_players)."<\/a><\/td><td>$kills<\/td>";

                my $total = $kills + $deaths + $suicides;
                my $skill = $kills - $deaths - $suicides;

                if ($total > 0 && $kills > 0) {
                    $img_width = ( 100 * $kills ) / $total;
                    $ratio = sprintf "%.2f%%", ($img_width);
                } else {
                    $ratio = "0.00%%";
                    $img_width = "1";
                }
                $playerstring .= "<td>$deaths<\/td><td>$suicides<\/td><td>$skill<\/td>\n";

                my $seconds = $totalTime{$top_players};
                $seconds = 0 if (!defined($totalTime{$top_players}));
                my $hours = int($seconds / 3600);      #convert to hours
                $seconds %= 3600;               #remove the hours we figured out
                $playerstring .= "<td>";
                $playerstring .= sprintf "%3dh%4dm%02ds",($hours),int($seconds/60),($seconds%60);
                $playerstring .= "<\/td>\n"; 

                $playerstring .= "
                <td><img src=\"$IMAGESDIR\/left.gif\" height=\"10\"><img src=\"$IMAGESDIR\/main.gif\" width=\"$img_width\" height=\"10\"><img src=\"$IMAGESDIR\/right.gif\" height=\"10\">&nbsp;&nbsp;&nbsp;$ratio<\/td>\n
                <\/tr>\n";

                # save this player for individual stats later
                $individuals{$top_players} = 1;
                $count++;
            } else {
                next;
            }
        }
        $count = 1;
        $playerstring .= "<\/table>\n";
        return ($playerstring);
    }


    if ($stat_type == 4) {  # Sort by Efficiency Ratio
        $playerstring .= "<td class=\"header\"><strong>Player Name<\/strong><\/td>
                 <td class=\"header\"><strong><a href=\"$OUTPUT_FILENAME\" class=\"player\"># Kills<\/a><\/strong><\/td>
                 <td class=\"header\"><strong><a href=\"$CONF_MAP{2}\" class=\"player\"># Deaths<\/a><\/strong><\/td>
                 <td class=\"header\"><strong><a href=\"$CONF_MAP{3}\" class=\"player\">Suicides<\/a><\/strong><\/td>
                 <td class=\"header\"><strong><a href=\"$CONF_MAP{1}\" class=\"player\">Skill<\/a><\/strong><\/td>
                 <td class=\"header\"><strong><a href=\"$CONF_MAP{5}\" class=\"player\">Time<\/a><\/strong><\/td>
                 <td class=\"header\"><strong>Efficiency Ratio<\/strong><\/td>\n
            <\/tr>\n
            ";     
        foreach my $top_players (sort { $by_eff{$b} <=> $by_eff{$a} } keys %by_eff) {
            if ($count != $TOPNUM + 1) {
                my $kills = $players{$top_players};
                my $suicides = $suicides{$top_players};
                my $deaths = $victim{$top_players};
        
                $kills = 0 if (!defined($players{$top_players}));
                $suicides = 0 if (!defined($suicides{$top_players}));
                $deaths = 0 if (!defined($victim{$top_players}));
        
                my $img_width;
                my $ratio;
                my $individual_player = $top_players;
                $individual_player =~ s/(\W)//g;

                $playerstring .= "<tr class=\"".alt_colors()."\">\n
                <td>$count\. <a href=\"$PLAYERDATADIR/$individual_player.html\" 
                class=\"player-stat-link\">".strip_colors($top_players).
                "<\/a><\/td><td>$kills<\/td>";

                my $total = $kills + $deaths + $suicides;
                my $skill = $kills - $deaths - $suicides;
        
                if ($total > 0 && $kills > 0) {
                    $img_width = ( 100 * $kills ) / $total;
                    $ratio = sprintf "%.2f%%", ($img_width);
                } else {
                    $ratio = "0.00%%";
                    $img_width = "1";
                }
                $playerstring .= "<td>$deaths<\/td><td>$suicides<\/td><td>$skill<\/td>\n";

                my $seconds = $totalTime{$top_players};
                $seconds = 0 if (!defined($totalTime{$top_players}));
                my $hours = int($seconds / 3600);      #convert to hours
                $seconds %= 3600;               #remove the hours we figured out
                $playerstring .= "<td>";
                $playerstring .= sprintf "%3dh%4dm%02ds",($hours),int($seconds/60),($seconds%60);
                $playerstring .= "<\/td>\n"; 

                $playerstring .= "
                <td><img src=\"$IMAGESDIR\/left.gif\" height=\"10\"><img src=\"$IMAGESDIR\/main.gif\" width=\"$img_width\" height=\"10\"><img src=\"$IMAGESDIR\/right.gif\" height=\"10\">&nbsp;&nbsp;&nbsp;$ratio<\/td>\n
                <\/tr>\n";

                # save this player for individual stats later
                $individuals{$top_players} = 1;
                $count++;
            } else {
                next;
            }
        }
        $count = 1;
        $playerstring .= "<\/table>\n";
        return ($playerstring);
    }

    if ($stat_type == 5) {  # Sort by time played
         $playerstring .= "<td class=\"header\"><strong>Player Name<\/strong><\/td>
                 <td class=\"header\"><strong><a href=\"$OUTPUT_FILENAME\" class=\"player\"># Kills<\/a><\/strong><\/td>
                 <td class=\"header\"><strong><a href=\"$CONF_MAP{2}\" class=\"player\"># Deaths<\/a><\/strong><\/td>
                 <td class=\"header\"><strong><a href=\"$CONF_MAP{3}\" class=\"player\">Suicides<\/a><\/strong><\/td>
                 <td class=\"header\"><strong><a href=\"$CONF_MAP{1}\" class=\"player\">Skill<\/a><\/strong><\/td>
                 <td class=\"header\"><strong>Time<\/strong><\/td>
                 <td class=\"header\"><strong><a href=\"$CONF_MAP{4}\" class=\"player\">Efficiency Ratio<\/a><\/strong><\/td>\n
                <\/tr>\n";     
        foreach my $top_players (sort { $totalTime{$b} <=> $totalTime{$a} } keys %totalTime) {
            if ($count != $TOPNUM + 1) {
                my $kills = $players{$top_players};
                my $suicides = $suicides{$top_players};
                my $deaths = $victim{$top_players};

                $kills = 0 if (!defined($players{$top_players}));
                $suicides = 0 if (!defined($suicides{$top_players}));
                $deaths = 0 if (!defined($victim{$top_players}));

                my $img_width;
                my $ratio;
                my $individual_player = $top_players;
                $individual_player =~ s/(\W)//g;

                $playerstring .= "<tr class=\"".alt_colors()."\">\n
                <td>$count\. <a href=\"$PLAYERDATADIR/$individual_player.html\" 
                class=\"player-stat-link\">".strip_colors($top_players).
                "<\/a><\/td><td>$kills<\/td>";

                my $total = $kills + $deaths + $suicides;
                my $skill = $kills - $deaths - $suicides;

                if ($total > 0 && $kills > 0) {
                    $img_width = ( 100 * $kills ) / $total;
                    $ratio = sprintf "%.2f%%", ($img_width);
                } else {
                    $ratio = "0.00%%";
                    $img_width = "1";
                }
                $playerstring .= "<td>$deaths<\/td><td>$suicides<\/td><td>$skill<\/td>\n";

                my $seconds = $totalTime{$top_players};
                $seconds = 0 if (!defined($totalTime{$top_players}));
                my $hours = int($seconds / 3600);      #convert to hours
                $seconds %= 3600;               #remove the hours we figured out
                $playerstring .= "<td>";
                $playerstring .= sprintf "%3dh%4dm%02ds",($hours),int($seconds/60),($seconds%60);
                $playerstring .= "<\/td>\n"; 

                $playerstring .= "
                <td><img src=\"$IMAGESDIR\/left.gif\" height=\"10\"><img src=\"$IMAGESDIR\/main.gif\" width=\"$img_width\" height=\"10\"><img src=\"$IMAGESDIR\/right.gif\" height=\"10\">&nbsp;&nbsp;&nbsp;$ratio<\/td>\n
                <\/tr>\n";

                # save this player for individual stats later
                $individuals{$top_players} = 1;
                $count++;
            } else {
                next;
            }
        }
        $count = 1;
        $playerstring .= "<\/table>\n";
        return ($playerstring);
    }
}

#######################
# Most Popular Weapon #
#######################
sub popular_weapon {
    my $pop_weapon = "
    <P>&nbsp;\n
    <table border=\"1\" cellspacing=\"0\" cellpadding=\"2\" width=\"50%\">\n
    <tr>\n
    <td class=\"header\" colspan=\"4\"><center>Most popular weapons</center><\/td>\n
    <tr>\n
    <\/tr>\n
    <tr>\n
    <td class=\"header\"><strong>Weapon<\/strong><\/td><td class=\"header\"><strong># Kills<\/strong><\/td><td class=\"header\"><strong>Usage Ratio<\/strong><\/td>\n
    <\/tr>\n";
    foreach my $weapon_love (sort { $weapon{$b} <=> $weapon{$a} } keys %weapon) {
        my $img_width = (100 * $weapon{$weapon_love} / $totalKills);
        my $weapon_ratio = sprintf "%.2f%%", ($img_width);
        $WEAPON_MAP{$weapon_love} = 0 unless (defined($WEAPON_MAP{$weapon_love}));
        $weapon{$weapon_love} = 0 unless (defined($weapon{$weapon_love}));
        $pop_weapon .= "<tr class=\"".alt_colors()."\">\n
        <td>$WEAPON_MAP{$weapon_love}<\/td><td>$weapon{$weapon_love}<\/td>
        <td>
        <img src=\"$IMAGESDIR/left.gif\" height=\"10\"><img src=\"$IMAGESDIR/main.gif\" height=\"10\" width=\"$img_width\"><img src=\"$IMAGESDIR/right.gif\" height=\"10\">&nbsp;&nbsp;&nbsp;$weapon_ratio<\/td>\n
        <\/tr>\n";
    }
    $pop_weapon .= "<\/table>\n";
    return ($pop_weapon);
}

#######################
# Top Weapon Handlers #
#######################
sub top_weapon_handler {
    my ($weap_hand);
    $count = 1;
    foreach my $Weapon (sort keys %WEAPON_MAP) {
        my $weap_new;
        if ($WEAPONS_TO_DISPLAY =~ /all/i || $Weapon =~ /$WEAPONS_TO_DISPLAY/) {
            $weap_new .= "
            <P>&nbsp;\n
            <table border=\"1\" cellspacing=\"0\" cellpadding=\"2\" width=\"80%\">\n
            <tr>\n
            <td class=\"header\" colspan=\"4\"><center>Top $TOPWEAP \(<strong>$WEAPON_MAP{$Weapon}<\/strong>) - Weapon Handlers<\/center><\/td>\n
            </tr>
            <tr>\n
            <td class=\"header\"><strong>Player<\/strong><\/td><td class=\"header\"><strong>Weapon<\/strong><\/td><td class=\"header\"><strong># Kills<\/strong><\/td><td class=\"header\"><strong>Ratio of total kills<\/strong><\/td>\n
            </tr>\n";
            foreach my $weapon_lovers (sort { $top_player_weapon{$b} <=> $top_player_weapon{$a} } keys %top_player_weapon) {
                if ($count != (($TOPWEAP) + 1)) {
                    my ($plyr, $weap) = split(/\t/, $weapon_lovers);
                    if ($weap eq $Weapon) {
                        my ($img_width, $pw_ratio);
                        if ($weapon{$weap} == 0) {
                            $img_width = 1;
                            $pw_ratio = "0.00%%";
                        }
                        else {
                            $img_width = (100 * $top_player_weapon{$weapon_lovers} / $weapon{$weap});
                            $pw_ratio = sprintf ("%.2f%%", ($img_width));
                        }
                        $weap_new .= "<tr class=\"".alt_colors()."\">\n
                                <td>".strip_colors($plyr)."<\/td><td>$WEAPON_MAP{$weap}<\/td><td>$top_player_weapon{$weapon_lovers}<\/td><td><img src=\"$IMAGESDIR/left.gif\" height=\"10\"><img src=\"$IMAGESDIR/main.gif\" height=\"10\" width=\"$img_width\"><img src=\"$IMAGESDIR/right.gif\" height=\"10\">&nbsp;&nbsp;&nbsp;$pw_ratio<\/td>\n
                                <\/tr>\n";
                        $count++;
                    }
                }
            }       
        }   
        $weap_new .= "<\/table>\n";
        #ignore weapons that have no kills
        if ($count != 1) { $weap_hand .= $weap_new; } ;
        $count = 1;
    }
    return($weap_hand);
}

##########################################
# Most Popular Target - Individual Stats #
##########################################
sub I_player_stats {
    my $player_name = shift;
    my $player_stats = "
    <P>&nbsp;\n
        <table border=\"1\" cellspacing=\"0\" cellpadding=\"2\" width=\"80%\">\n
    <tr>\n
        <td class=\"header\" colspan=\"6\"><center>".strip_colors($player_name)."'s Player Statistics</center><\/td>\n
    <\/tr>\n
    <tr>\n
    <td class=\"header\"><strong># Kills<\/strong><\/td>
    <td class=\"header\"><strong># Deaths<\/strong><\/td>
    <td class=\"header\"><strong># Suicides<\/strong><\/td>
    <td class=\"header\"><strong>Skill<\/strong><\/td>
    <td class=\"header\"><strong>Time<\/strong><\/td>
    <td class=\"header\"><strong>Ratio<\/strong><\/td>\n
    <\/tr>\n";

    my $kills = $players{$player_name};
    my $suicides = $suicides{$player_name};
    my $deaths = $victim{$player_name};

    $kills = 0 if (!defined($players{$player_name}));
    $suicides = 0 if (!defined($suicides{$player_name}));
    $deaths = 0 if (!defined($victim{$player_name}));

    my $img_width;
    my $ratio;
    my $total = $kills + $deaths + $suicides;
    my $skill = $kills - $deaths - $suicides;

    if ($total > 0 && $kills > 0) {
        $img_width = ( 100 * $kills ) / $total;
        $ratio = sprintf "%.2f%%", ($img_width);
    } else {
        $ratio = "0.00%%";
        $img_width = "1";
    }
    $player_stats .= "<tr class=\"".alt_colors()."\">\n
    <td>$kills<\/td><td>$deaths<\/td><td>$suicides<\/td><td>$skill<\/td>";

    my $seconds = $totalTime{$player_name};
    $seconds = 0 if (!defined($totalTime{$player_name}));
    my $hours = int($seconds / 3600);      #convert to hours
    $seconds %= 3600;               #remove the hours we figured out
    $player_stats .= "<td>";
    $player_stats .= sprintf "%3dh%4dm%02ds",($hours),int($seconds/60),($seconds%60);
    $player_stats .= "<\/td>\n"; 

    $player_stats .= "
    <td><img src=\"../$IMAGESDIR\/left.gif\" height=\"10\"><img src=\"../$IMAGESDIR\/main.gif\" width=\"$img_width\" height=\"10\"><img src=\"../$IMAGESDIR\/right.gif\" height=\"10\">&nbsp;&nbsp;&nbsp;$ratio<\/td>\n
    <\/tr>\n
    <\/table>\n";
    return($player_stats);
}


##########################################
# Most Popular Target - Individual Stats #
##########################################
sub I_player_vs_player {
    my $player_name = shift;
    my $pop_victim = "
    <P>&nbsp;\n
        <table border=\"1\" cellspacing=\"0\" cellpadding=\"2\" width=\"80%\">\n
    <tr>\n
    <td class=\"header\" colspan=\"4\"><center>".strip_colors($player_name)."'s Favorite Targets<\/center><\/td>\n
    <\/tr>\n
    <tr>\n
    <td class=\"header\"><strong>Victim<\/strong><\/td><td class=\"header\"><strong># Kills<\/strong><\/td><td class=\"header\">
    <strong>Ratio of total Kills<\/strong><\/td>\n
    <\/tr>\n";
    foreach my $player_love ( sort { $player_vs_player{$b} <=> $player_vs_player{$a} } keys %player_vs_player) {
        if ($count != $TOPWEAP + 1) {
            my ($plyr, $victim) = split(/\t/, $player_love);
            if ($plyr eq $player_name) {
                my ($img_width, $pw_ratio);
                if ($players{$plyr} == 0) {
                    $img_width = 1;
                    $pw_ratio = "0.00%%";
                }
                else {
                    $img_width = (100 * $player_vs_player{$player_love} / $players{$plyr});
                    $pw_ratio = sprintf "%.2f%%", ($img_width);
                }
                $pop_victim .= "<tr class=\"".alt_colors()."\">\n
                <td>".strip_colors($victim)."<\/td><td>$player_vs_player{$player_love}<\/td><td><img src=\"../$IMAGESDIR/left.gif\" height=\"10\"><img src=\"../$IMAGESDIR/main.gif\" height=\"10\" width=\"$img_width\"><img src=\"../$IMAGESDIR/right.gif\" height=\"10\">&nbsp;&nbsp;&nbsp;$pw_ratio<\/td>\n
                <\/tr>\n";
                $count++;
            }
        } else {
            next;
        }
    }
    $count = 1;
    $pop_victim .= "<\/table>\n";
    return($pop_victim);
}
    
##########################################
# Most Popular Weapon - Individual Stats #
##########################################
sub I_popular_weapon {
    my $player_name = shift;
    my $pop_weapon = "
    <P>&nbsp;\n
        <table border=\"1\" cellspacing=\"0\" cellpadding=\"2\" width=\"80%\">\n
    <tr>\n
        <td class=\"header\" colspan=\"4\"><center>".strip_colors($player_name)."'s Favorite Weapons</center><\/td>\n
    <\/tr>\n
    <tr>\n
    <td class=\"header\"><strong>Weapon<\/strong><\/td><td class=\"header\"><strong># Kills<\/strong><\/td><td class=\"header\">
<strong>Ratio of total Kills<\/strong><\/td>\n
    <\/tr>\n";
    foreach my $weapon_lovers ( sort { $top_player_weapon{$b} <=> $top_player_weapon{$a} } keys %top_player_weapon) {
        my ($plyr, $weap) = split(/\t/, $weapon_lovers);
        if ($plyr eq $player_name) {
            if ($plyr eq $player_name) {
                my ($img_width, $pw_ratio);
                if (!defined($players{$plyr}) || $players{$plyr} == 0) {
                    $img_width = 1;
                    $pw_ratio = "0.00%%";
                }
                else {
                    $img_width = (100 * $top_player_weapon{$weapon_lovers} / $players{$plyr});
                    $pw_ratio = sprintf "%.2f%%", ($img_width);
                }
                $pop_weapon .= "<tr class=\"".alt_colors()."\">\n
                    <td>$WEAPON_MAP{$weap}<\/td><td>$top_player_weapon{$weapon_lovers}<\/td><td><img src=\"../$IMAGESDIR/left.gif\" height=\"10\"><img src=\"../$IMAGESDIR/main.gif\" height=\"10\" width=\"$img_width\"><img src=\"../$IMAGESDIR/right.gif\" height=\"10\">&nbsp;&nbsp;&nbsp;$pw_ratio<\/td>\n
                    <\/tr>\n";
            }
        }
    }
    $pop_weapon .= "<\/table>\n";
    return ($pop_weapon);
}


###################
sub strip_colors  # Convert the Q3 color control codes to HTML format
################### Thanks to Jim Walsh for fixing the colorizer!!!!
{
    my $player_name = shift;
    my $name_count = 0;
    my $i = 0;
    #$name_count++ while($player_name =~ /\^/g);

    # fix up names with non html compatible characters
    $player_name =~ s/\</\&lt\;/g;       
    $player_name =~ s/\>/\&gt\;/g; 
    $player_name =~ s/\</\&lt\;/g;
    $player_name =~ s/\>/\&gt\;/g;

    for ($i = 0; $i <= 7; $i++) {
        #dunno what the $2 did, but it was causing a warning
        #$player_name =~ s/\^$i/<font color=\"$COLOR_MAP{$i}\">$2/g;
        $player_name =~ s/\^$i/<font color=\"$COLOR_MAP{$i}\">/g;
    }
    
    $name_count = 0;
    # Find the characters that haven't been converted yet ; convert them to their octal format

    $name_count++ while($player_name =~ /\^/g);
    
    while ($player_name =~ /\^/g) {
        if ($DEBUG) { print "$name_count - Player Name: $player_name\n"; };
        my $tmpPlayer = $1 if $player_name =~ /\^(.)/;
        my $octColor = sprintf "%o", ord $tmpPlayer;
        $octColor = chop($octColor);
        if ($DEBUG) {
            print "Converting $tmpPlayer to Octal: $octColor\n";
        }
        $player_name =~ s/\^\Q$1\E/<font color\"$COLOR_MAP{$octColor}\">/g;
        if ($DEBUG) {
            print "Player Name is now:  $player_name \n";
        }
    }
    # Lastly, if we didn't catch it, strip it out
    $player_name =~ s/\^//g;
        
    
    $player_name .= "<\/font>";
    return ($player_name);
}
    
################
sub alt_colors #
################
{
    if ($alt == 0) {
        $alt = 1;
        return("cell1");
    } else {
        $alt = 0;
        return("cell2");
    }
}

###################
sub init_datafile #
###################
{
    open (DATAFILE, ">$DATADIR/players.dat") or die("Could not initialize the player datafile ($DATADIR/players.dat)\nError: $!");
    print DATAFILE "Init: ". scalar localtime();
    close(DATAFILE);
}


__END__

=head1 NAME

    logparsesof2.pl - Soldier of Fortune 2 Server Log Processing Program

=head1 PREREQUISITES

    Perl 5.003 or greater

=head1 PUBLIC INTERFACE
    

    logparse.pl 
           -c Specify which config file the script should use 

           -reinit
           Use this option to reinitialize (reset) the player datafile

           -logfile <path and filename>
           Use this option to specify an alternate logfile.  This will 
           override the LOGFILE directive in the .conf file

           -h
           Use this option to display this screen.  (Help Screen)

           -d
           Enable debug mode from the command line 
           (lots of output, you are warned!)

=head1 AUTHORS

Sam Evans (sam@neuroflux.com)

Jesse Brandeburg '{LLC}miCro' (go_jesse@yahoo.com)

Ravencroft '[NN]-=RC=-' (ravencroft@numbnutz.net)

=head1 HOMEPAGE

http://jk2.gamersimpact.com
